@@ -452,7 +452,7 @@ register int len;
       /* The next part is only for speedup */
       if (curr->w_state == LIT &&
 #ifdef KANJI
-          curr->w_Font != KANJI && curr->w_Font != KANA &&
+          curr->w_Font != KANJI && curr->w_Font != KANA && !curr->w_mbcs &&
 #endif
           c >= ' ' && 
           ((c & 0x80) == 0 || ((c >= 0xa0 || !curr->w_c1) && !curr->w_gr && (display == 0 || !D_CB8))) &&
