@@ -359,7 +359,7 @@ FinitTerm()
       ResizeDisplay(D_defwidth, D_defheight);
       DefRestore();
       SetAttrFont(0, ASCII);
-      if (D_HS)
+      if (D_hstatus)
 	PutStr(D_DS);
       D_x = D_y = -1;
       GotoPos(0, D_height - 1);
@@ -1284,11 +1284,12 @@ char *msg;
 	  debug("using HS\n");
           SetAttrFont(0, ASCII);
 	  InsertMode(0);
-	  if (D_fore && D_fore->w_hstatus)
+	  if (D_hstatus)
 	    PutStr(D_DS);
 	  CPutStr(D_TS, 0);
 	  AddStr(msg);
 	  PutStr(D_FS);
+	  D_hstatus = 1;
 	}
       Flush();
       (void) time(&D_status_time);
@@ -1337,20 +1338,26 @@ RemoveStatus()
     {
       /*
       SetAttrFont(0, ASCII);
-      PutStr(D_DS);
+      if (D_hstatus)
+        PutStr(D_DS);
       */
       RefreshStatus();
     }
   SetCursor();
 }
 
+/*
+ *  Refreshes the harstatus of the _window_. Shouldn't be here...
+ */
+
 void
 RefreshStatus()
 {
   if (D_HS)
     {
       SetAttrFont(0, ASCII);
-      PutStr(D_DS);
+      if (D_hstatus)
+        PutStr(D_DS);
       if (D_fore && D_fore->w_hstatus)
 	{
 	  CPutStr(D_TS, 0);
@@ -1359,6 +1366,7 @@ RefreshStatus()
 	  else
 	    AddStr(D_fore->w_hstatus);
 	  PutStr(D_FS);
+	  D_hstatus = 1;
 	}
     }
   else if (D_fore && D_fore->w_hstatus)
@@ -1774,6 +1782,8 @@ NukePending()
   D_font = ASCII;
   D_attr = 0;
   D_atyp = 0;
+  PutStr(D_DS);
+  D_hstatus = 0;
   ChangeScrollRegion(oldtop, oldbot);
   SetAttrFont(oldattr, oldfont);
   KeypadMode(oldkeypad);
