@@ -506,8 +506,11 @@ int dump;
 #ifdef COPY_PASTE
 	    case DUMP_EXCHANGE:
 	      p = D_user->u_copybuffer;
-	      for (i = 0; i < D_user->u_copylen; i++)
-		putc(*p++, f);
+	      for (i = D_user->u_copylen; i-- > 0; p++)
+		if (*p == '\r' && (i == 0 || p[1] != '\n'))
+		  putc('\n', f);
+		else
+		  putc(*p, f);
 	      break;
 #endif
 	    }
@@ -549,9 +552,10 @@ char *fn;
 int *lenp;
 {
   int i, l, size;
-  char c, *buf;
+  char c, *bp, *buf;
   struct stat stb;
 
+  ASSERT(lenp);
   debug1("ReadFile(%s)\n", fn);
   if ((i = secopen(fn, O_RDONLY, 0)) < 0)
     {
@@ -583,16 +587,20 @@ int *lenp;
 #endif
       Msg(errno, "Got only %d bytes from %s", l, fn);
       close(i);
-      return NULL;
     }
-  if (read(i, &c, 1) > 0)
-    Msg(0, "Slurped only %d characters (of %d) into buffer - try again", 
-        l, size);
   else
-    Msg(0, "Slurped %d characters into buffer", l);
+    {
+      if (read(i, &c, 1) > 0)
+	Msg(0, "Slurped only %d characters (of %d) into buffer - try again", 
+	    l, size);
+      else
+	Msg(0, "Slurped %d characters into buffer", l);
+    }
   close(i);
-  if (lenp)
-    *lenp = l;
+  *lenp = l;
+  for (bp = buf; l-- > 0; bp++)
+    if (*bp == '\n' && (bp == buf || bp[-1] != '\r'))
+      *bp = '\r';
   return buf;
 }
 
