@@ -169,12 +169,20 @@
  *                     Ugly seteuid bug in ForkWindow() fixed.
  * 20.12.94 -- 3.06.00 Solaris has broken seteuid().
  *                     osf loadav patches.  -- DISTRIBUTED
+ * 16.01.95 -- 3.06.01 KANJI patch. doc/Makefile.in fixed.
+ *                     Install now calls doc/Makefile install.
+ *                     Don't use 'ds' too often, breaks MSkermit.
+ *                     undef'd LOGOUTOK logs in now.
+ *                     Ultrix is broken, too (seteuid).
+ *                     Use \r (not \n) to join lines in pastebuf.
+ *                     bindkey can now remove sequences.
+ *                     InitTTY fixed for PLAIN. -- DISTRIBUTED
  *                     
  */
 
 #define ORIGIN "FAU"
 #define REV 3
 #define VERS 6
-#define PATCHLEVEL 0
-#define DATE "20-Dec-94"
+#define PATCHLEVEL 1
+#define DATE "16-Jan-95"
 #define STATE "" 
