@@ -271,8 +271,6 @@ InitKeytab()
   ktab['C'].nr = RC_CLEAR;
   ktab['Z'].nr = RC_RESET;
   ktab['H'].nr = RC_LOG;
-  ktab[(int)(unsigned char)DefaultEsc].nr = RC_OTHER;
-  ktab[(int)(unsigned char)DefaultMetaEsc].nr = RC_META;
   ktab['M'].nr = RC_MONITOR;
   ktab['?'].nr = RC_HELP;
   for (i = 0; i < ((MAXWIN < 10) ? MAXWIN : 10); i++)
@@ -312,6 +310,8 @@ InitKeytab()
   ktab['b'].nr = ktab[Ctrl('b')].nr = RC_BREAK;
   ktab['B'].nr = RC_POW_BREAK;
   ktab['_'].nr = RC_SILENCE;
+  ktab[(int)(unsigned char)DefaultEsc].nr = RC_OTHER;
+  ktab[(int)(unsigned char)DefaultMetaEsc].nr = RC_META;
 }
 
 static void
@@ -1924,6 +1924,7 @@ int key;
 	  struct action *newact;
           int newnr, fl = 0, kf = 0, af = 0, df = 0, mf = 0;
 	  struct display *odisp = display;
+	  int used = 0;
 
 	  for (; *args && **args == '-'; args++)
 	    {
@@ -1965,8 +1966,6 @@ int key;
 	    }
 	  if (kf == 0)
 	    {
-	      int used = 0;
-
 	      if (af)
 		{
 		  Msg(0, "%s: bindkey: -a only works with -k", rc_name);
@@ -1992,12 +1991,6 @@ int key;
 		used = 1;
 	      if ((df || mf) && umtab[i + KMAP_KEYS + KMAP_AKEYS].nr != RC_ILLEGAL)
 		used = 1;
-	      if ((args[1] || !used) && kmap_extras[i])
-		{
-		  free(kmap_extras[i]);
-		  kmap_extras[i] = 0;
-		  kmap_extras_fl[i] = 0;
-		}
 	      i += KMAP_KEYS + KMAP_AKEYS;
 	    }
 	  else
@@ -2031,12 +2024,24 @@ int key;
 		newact->args = SaveArgs(args + 2);
 	      if (kf == 0 && args[1])
 		{
+		  if (kmap_extras[i - (KMAP_KEYS+KMAP_AKEYS)])
+		    free(kmap_extras[i - (KMAP_KEYS+KMAP_AKEYS)]);
 	          kmap_extras[i - (KMAP_KEYS+KMAP_AKEYS)] = SaveStr(*args);
 		  kmap_extras_fl[i - (KMAP_KEYS+KMAP_AKEYS)] = fl;
 		}
 	    }
 	  for (display = displays; display; display = display->d_next)
 	    remap(i, args[1] ? 1 : 0);
+	  if (kf == 0 && !args[1])
+	    {
+	      i -= KMAP_KEYS + KMAP_AKEYS;
+	      if (!used && kmap_extras[i])
+		{
+		  free(kmap_extras[i]);
+		  kmap_extras[i] = 0;
+		  kmap_extras_fl[i] = 0;
+		}
+	    }
 	  display = odisp;
 	}
       break;
