@@ -247,16 +247,21 @@ struct NewWindow *newwin;
   p->w_next = windows;
   windows = p;
 #ifdef UTMPOK
+# ifdef LOGOUTOK
   debug1("MakeWindow will %slog in.\n", nwin.lflag?"":"not ");
-  if (nwin.lflag == 1)
+  if (!nwin.lflag)
+    p->w_slot = (slot_t) -1;
+  else
+# else /* LOGOUTOK */
+  debug1("MakeWindow will log in, LOGOUTOK undefined in config.h%s.\n", 
+         nwin.lflag?"":" (although lflag=0)");
+# endif /* LOGOUTOK */
     {
       if (display)
         SetUtmp(p);
       else
 	p->w_slot = (slot_t) 0;
     }
-  else
-    p->w_slot = (slot_t) -1;
 #endif
   SetForeWindow(p);
   Activate(p->w_norefresh);
